% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CLUSTERING-compare-clusterings.R
\name{compare_clusterings_configs}
\alias{compare_clusterings_configs}
\title{Create clustering configurations.}
\usage{
compare_clusterings_configs(
  types = c("p", "h", "f"),
  k = 2L,
  controls = NULL,
  preprocs = pdc_configs("preproc", none = list()),
  distances = pdc_configs("distance", dtw_basic = list()),
  centroids = pdc_configs("centroid", default = list()),
  no.expand = character(0L)
)
}
\arguments{
\item{types}{Clustering types. It must be any combination of (possibly abbreviated): partitional,
hierarchical, fuzzy, tadpole.}

\item{k}{A numeric vector with one or more elements specifying the number of clusters to test.}

\item{controls}{A named list of \link{tsclust-controls}. \code{NULL} means defaults. See details.}

\item{preprocs}{Preprocessing configurations. See details.}

\item{distances}{Distance configurations. See details.}

\item{centroids}{Centroid configurations. See details.}

\item{no.expand}{A character vector indicating parameters that should \emph{not} be expanded between
\code{\link[=pdc_configs]{pdc_configs()}} configurations. See examples.}
}
\value{
A list for each clustering type, each of which includes a data frame with the computed and merged
configurations. Each data frame has an extra attribute \code{num.configs} specifying the number of
configurations.
}
\description{
Create configurations for \code{\link[=compare_clusterings]{compare_clusterings()}}
}
\details{
Preprocessing, distance and centroid configurations are specified with the helper function
\code{\link[=pdc_configs]{pdc_configs()}}, refer to the examples in \code{\link[=compare_clusterings]{compare_clusterings()}} to see how this is used.

The controls list may be specified with the usual \link{tsclust-controls} functions. The names of the
list must correspond to "partitional", "hierarchical", "fuzzy" or "tadpole" clustering. Again,
please refer to the examples in \code{\link[=compare_clusterings]{compare_clusterings()}}.
}
\examples{

# compare this with leaving no.expand empty
compare_clusterings_configs(
    distances = pdc_configs("d", dtw_basic = list(window.size = 1L:2L, norm = c("L1", "L2"))),
    centroids = pdc_configs("c", dba = list(window.size = 1L:2L, norm = c("L1", "L2"))),
    no.expand = c("window.size", "norm")
)

}
